#
# spec file for package VirtualGL
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           VirtualGL
Version:        3.1
Release:        lp160.1.3
Summary:        A toolkit for displaying OpenGL applications to thin clients
License:        LGPL-2.1-only AND SUSE-wxWidgets-3.1
Group:          Productivity/Networking/Other
URL:            http://www.virtualgl.org
Source0:        https://sourceforge.net/projects/virtualgl/files/%{version}/%{name}-%{version}.tar.gz
Source1:        baselibs.conf
Patch1:         VirtualGL-link-libs.patch
BuildRequires:  Mesa-devel
BuildRequires:  Mesa-libGLU-devel
BuildRequires:  cmake
BuildRequires:  fltk-devel
BuildRequires:  gcc-c++
BuildRequires:  glibc-devel
BuildRequires:  libjpeg-devel
BuildRequires:  opencl-headers
BuildRequires:  pkgconfig
BuildRequires:  pkgconfig(OpenCL)
BuildRequires:  pkgconfig(egl)
BuildRequires:  pkgconfig(gl)
BuildRequires:  pkgconfig(glproto)
BuildRequires:  pkgconfig(glu)
BuildRequires:  pkgconfig(x11)
BuildRequires:  pkgconfig(x11-xcb)
BuildRequires:  pkgconfig(xcb-keysyms)
BuildRequires:  pkgconfig(xext)
BuildRequires:  pkgconfig(xtst)
BuildRequires:  pkgconfig(xv)

%description
VirtualGL is a library which allows most Linux OpenGL applications to be
remotely displayed to a thin client without the need to alter the
applications in any way.  VGL inserts itself into an application at run time
and intercepts a handful of GLX calls, which it reroutes to the server's
display (which presumably has a 3D accelerator attached.)  This causes all
3D rendering to occur on the server's display.  As each frame is rendered
by the server, VirtualGL reads back the pixels from the server's framebuffer
and sends them to the client for re-compositing into the appropriate X
Window.  VirtualGL can be used to give hardware-accelerated 3D capabilities to
VNC or other remote display environments that lack GLX support.  In a LAN
environment, it can also be used with its built-in motion-JPEG video delivery
system to remotely display full-screen 3D applications at 20+ frames/second.

VirtualGL is based upon ideas presented in various academic papers on
this topic, including "A Generic Solution for Hardware-Accelerated Remote
Visualization" (Stegmaier, Magallon, Ertl 2002) and "A Framework for
Interactive Hardware Accelerated Remote 3D-Visualization" (Engel, Sommer,
Ertl 2000.)

%package devel
Summary:        A toolkit for displaying OpenGL applications to thin clients
Group:          Development/Libraries/Other
Requires:       %{name} = %{version}

%description devel
VirtualGL is a library which allows most Linux OpenGL applications to be
remotely displayed to a thin client without the need to alter the
applications in any way.  VGL inserts itself into an application at run time
and intercepts a handful of GLX calls, which it reroutes to the server's
display (which presumably has a 3D accelerator attached.)  This causes all
3D rendering to occur on the server's display.  As each frame is rendered
by the server, VirtualGL reads back the pixels from the server's framebuffer
and sends them to the client for re-compositing into the appropriate X
Window.  VirtualGL can be used to give hardware-accelerated 3D capabilities to
VNC or other remote display environments that lack GLX support.  In a LAN
environment, it can also be used with its built-in motion-JPEG video delivery
system to remotely display full-screen 3D applications at 20+ frames/second.

VirtualGL is based upon ideas presented in various academic papers on
this topic, including "A Generic Solution for Hardware-Accelerated Remote
Visualization" (Stegmaier, Magallon, Ertl 2002) and "A Framework for
Interactive Hardware Accelerated Remote 3D-Visualization" (Engel, Sommer,
Ertl 2000.)

%prep
%autosetup -p1

# Use /var/lib
sed -e "s#%{_sysconfdir}/opt#%{_localstatedir}/lib#g" \
    -i doc/unixconfig.txt doc/index.html \
    -i doc/advancedopengl.txt \
    -i server/vglrun.in \
    -i server/vglgenkey \
    -i server/vglserver_config

%build
%cmake \
    -DVGL_SYSTEMFLTK=ON \
    -DVGL_SYSTEMGLX=ON \
    -DVGL_SYSTEMXCB=ON \
    -DVGL_USESSL=OFF \
    -DVGL_USEXV=ON \
    -DTJPEG_INCLUDE_DIR=%{_includedir} \
    -DVGL_LIBDIR=%{_libdir} \
    -DTJPEG_LIBRARY=%{_libdir}/libturbojpeg.so \
    -DCMAKE_LIBRARY_PATH=%{_libdir}
make %{?_smp_mflags}

%install
%cmake_install
# Fix placement of 64b glxspheres
%ifarch x86_64 s390x ppc64 ppc64le aarch64 riscv64
mv %{buildroot}%{_bindir}/glxspheres* %{buildroot}%{_bindir}/glxspheres
mv %{buildroot}%{_bindir}/eglxspheres* %{buildroot}%{_bindir}/eglxspheres
%endif
# Fix fakelib placement
rm -rf %{buildroot}%{_prefix}/fakelib
mkdir -p %{buildroot}%{_libdir}/fakelib
ln -sf ../libvglfaker.so %{buildroot}%{_libdir}/fakelib/libGL.so
# Docs had wrong names
chmod 644 doc/LGPL.txt doc/LICENSE.txt doc/index.html doc/*.png doc/*.gif doc/*.css
rm -rf %{buildroot}%{_datadir}/doc
mv -f %{buildroot}%{_bindir}/glxinfo %{buildroot}%{_bindir}/vglxinfo
mv -f %{buildroot}%{_bindir}/eglinfo %{buildroot}%{_bindir}/veglinfo
mv -f %{buildroot}%{_bindir}/eglxinfo %{buildroot}%{_bindir}/veglxinfo
# bsc#1097210 rely on env settings
rm -rf %{buildroot}/%{_bindir}/.vglrun.*

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%doc LGPL.txt LICENSE.txt ChangeLog.md doc/index.html doc/legalinfo.txt doc/*.png doc/*.gif doc/*.css
%dir %{_libdir}/fakelib
%{_bindir}/tcbench
%{_bindir}/nettest
%{_bindir}/cpustat
%{_bindir}/veglinfo
%{_bindir}/vglxinfo
%{_bindir}/veglxinfo
%{_bindir}/glxspheres
%{_bindir}/eglxspheres
%{_bindir}/vglclient
%{_bindir}/vglconfig
%{_bindir}/vglconnect
%{_bindir}/vglgenkey
%{_bindir}/vgllogin
%{_bindir}/vglserver_config
%{_bindir}/vglrun
%{_bindir}/glreadtest
%{_libdir}/fakelib/libGL.so
%{_libdir}/libvglfaker.so
%{_libdir}/libdlfaker.so
%{_libdir}/libvglfaker-nodl.so
%{_libdir}/libvglfaker-opencl.so
%{_libdir}/libgefaker.so

%files devel
%{_includedir}/rrtransport.h
%{_includedir}/rr.h

%changelog
* Wed May 15 2024 Dominique Leuenberger <dimstar@opensuse.org>
- Use %%autosetup macro: allows us to eliminate usage of deprecated
  %%patchN syntax.
* Wed Dec  6 2023 ming li <mli@suse.com>
- Update to 3.1:
  1. Fixed an issue in the EGL back end whereby textures and other OpenGL objects
  were not automatically destroyed along with the context and drawable in which
  the objects were created.
  2. Added an environment variable (`VGL_EXITFUNCTION`) that, when set to
  `_exit` or `abort`, causes the VirtualGL Faker to call the specified function
  rather than `exit()` when a non-recoverable error occurs.
  3. Fixed an issue whereby the interposed `eglCreatePlatformWindowSurface()`
  and `eglCreatePlatformWindowSurfaceEXT()` functions incorrectly treated the
  native window argument as an X window handle rather than a pointer to an X
  window handle.  This caused a segfault in VLC when using the OpenGL video
  output module.
  4. `vglserver_config` now works properly with SUSE Linux Enterprise/openSUSE
  Leap 15.
  5. If the GLX back end is in use, then the interposed `eglGetDisplay()` and
  `eglGetPlatformDisplay()` functions now return `EGL_NO_DISPLAY` rather than
  throwing a fatal error.  This allows applications such as Firefox to fail
  gracefully or to fall back and use the GLX API if EGL/X11 is unavailable.
  6. Fixed an issue whereby the VirtualGL Configuration dialog did not pop up if
  the X keyboard extension was enabled on the 2D X server.
  7. The VirtualGL Faker no longer probes the 2D X server for stereo visuals
  unless the VGL Transport or a transport plugin will be used.  Even if the 2D X
  server has stereo visuals, they will never be used with the X11 and XV
  Transports.  Probing the 2D X server for stereo visuals causes problems with
  certain OpenGL implementations and with applications, such as Tecplot 360, that
  include static builds of Mesa.  An undocumented environment variable
  (`VGL_PROBEGLX`) can be used to override the default behavior.
  8. When using the EGL back end, interposed `XQueryExtension(..., "GLX", ...)`
  and `glXQueryExtension()` function calls now return `False`, rather than
  throwing a fatal error, if the 2D X server does not have a GLX extension.  (The
  EGL back end uses the 2D X server's GLX extension for GLX error handling.)
  This allows applications, such as Chrome/Chromium, to fail gracefully or use a
  different API (such as EGL/X11) if the VirtualGL Faker is unable to emulate
  GLX.
  9. The VirtualGL Client now runs on Macs with Apple silicon CPUs (without
  requiring Rosetta 2.)
- update to 3.0.90 (3.1 beta1)
  1. The `vglconnect -x` and `vglconnect -k` options have been retired in this
  version of VirtualGL and will continue to be maintained in the 3.0.x branch on
  a break/fix basis only.  Those options, which had been undocumented since
  VirtualGL 2.6.1, were a throwback to the early days of VirtualGL when SSH was
  not universally available and SSH X11 forwarding sometimes introduced a
  performance penalty.  `vglconnect -x` did not work with most modern operating
  systems, since most modern operating systems disable X11 TCP connections by
  default.
  2. SSL encryption of the VGL Transport has been retired in this version of
  VirtualGL and will continue to be maintained in the 3.0.x branch on a break/fix
  basis only.  That feature, which had not been included in official VirtualGL
  packages since VirtualGL 2.1.x, was not widely used.  Furthermore, SSL
  encryption of the VGL Transport had no performance advantages over SSH
  tunneling on modern systems, and it had some security disadvantages due to its
  reliance on the RSA key exchange algorithm (which made it incompatible with TLS
  v1.3.)
  3. When using the EGL back end, VirtualGL now supports 3D applications
  (including recent versions of Google Chrome/Chromium and Mozilla Firefox) that
  use the EGL/X11 API instead of the GLX API.  As of this writing, VirtualGL does
  not support EGL pixmap surfaces or front buffer/single buffer rendering with
  EGL window surfaces.
  4. On Un*x and Mac clients, `vglconnect` now uses the OpenSSH `ControlMaster`
  option to avoid the need to authenticate with the server multiple times when
  tunneling the VGL Transport through SSH.
  5. `vglconnect` now accepts a new command-line argument (`-v`) that, when
  combined with `-s`, causes VirtualGL to be preloaded into all processes
  launched in the remote shell, thus eliminating the need to invoke `vglrun`.
* Wed Dec 21 2022 Dirk Müller <dmueller@suse.com>
- update to 3.0.2:
  1. Support for transparent overlay visuals has been retired in this version of
  VirtualGL.  That feature will continue to be maintained in the 2.6.x branch on
  a break/fix basis only.  Most applications that once used transparent overlay
  visuals used them with color index rendering, which was removed in OpenGL 3.1
  in 2009.  Thus, almost all applications that render overlays now do so using
  other mechanisms.  Furthermore, the need for VirtualGL to hand off the
  rendering of transparent overlay visuals to the 2D X server has always limited
  the usefulness of the feature, and the discontinuation of the VirtualGL Client
  for Exceed relegated the feature to Un*x clients (with workstation-class GPUs)
  and the VGL Transport only.  Given that nVidia's implementation of transparent
  overlay visuals requires disabling the X Composite extension, which cannot be
  done in many modern Linux distributions, that further limited the feature to
  the point of uselessness.
  2. The VirtualGL Faker now assigns various permutations of common OpenGL
  rendering attributes to the available 2D X server visuals.  This maximizes the
  chances that "visual hunting" 3D applications (applications that use X11
  functions to obtain a list of 2D X server visuals, then iterate through the
  list with `glXGetConfig()` until they find a visual with a desired set of
  OpenGL rendering attributes) will find a suitable visual.
  `VGL_DEFAULTFBCONFIG` can still be used to assign a specified set of OpenGL
  rendering attributes to all 2D X server visuals, although the usefulness of
  that feature is now very limited.
  3. The VirtualGL Faker now includes an EGL back end that optionally emulates
  the GLX API using a combination of the EGL API (with the
  `EGL_EXT_platform_device` extension) and OpenGL renderbuffer objects (RBOs.)
  On supported platforms, the EGL back end allows the VirtualGL Faker to be used
  without a 3D X server.  The EGL back end can be activated by setting the
  `VGL_DISPLAY` environment variable to the path of a DRI device, such as
  * */dev/dri/card0**, or by passing that device path to `vglrun` using the `-d`
  argument.  Some obsolete OpenGL and GLX features are not supported by the EGL
* Sun Dec 11 2022 Anonymous Checkouts <anoncvs@mailinator.com>
- update to 3.0.2
  * Now supports use of EGL as well as GLX
* Sat Dec 19 2020 Dirk Müller <dmueller@suse.com>
- update to 2.6.5:
  1. Fixed a race condition that sometimes caused various fatal errors in the
  interposed `glXMakeContextCurrent()` function if both GLX drawable IDs passed
  to that function were the same window handle and the corresponding X window was
  simultaneously resized in another thread.
  2. Fixed an oversight whereby the addresses of the interposed
  `glDrawBuffers()`, `glGetString()`, and `glGetStringi()` functions introduced
  in 2.6.3[2] and 2.6.4[1] were not returned from the interposed
  `glXGetProcAddress()` and `glXGetProcAddressARB()` functions.
  3. VirtualGL now works properly with 3D applications that use the
  `glNamedFramebufferDrawBuffer()` and `glNamedFramebufferDrawBuffers()`
  functions (OpenGL 4.5) or the `glFramebufferDrawBufferEXT()` and
  `glFramebufferDrawBuffersEXT()` functions (`GL_EXT_direct_state_access`) and
  render to the front buffer.
  4. Fixed a BadRequest X11 error that occurred when attempting to use the X11
  Transport with a remote X connection.
  5. Worked around an issue with certain GLX implementations that list
  10-bit-per-component FB configs ahead of 8-bit-per-component FB configs and
  incorrectly set `GLX_DRAWABLE_TYPE|=GLX_PIXMAP_BIT` for those 10-bpc FB
  configs, even though they have no X visuals attached.  This caused VirtualGL's
  interposed `glXChooseVisual()` function to choose one of the 10-bpc FB configs
  behind the scenes, which made it impossible to use the VGL Transport.
  6. Fixed an issue whereby, when using the X11 Transport, a vertically flipped
  image of a previously-rendered frame was sometimes displayed if the 3D
  application called `glFlush()` while the front buffer was the active drawing
  buffer and the render mode was `GL_FEEDBACK` or `GL_SELECT`.
  7. `vglserver_config` now works properly if invoked with a relative path
  (for example, `cd /opt/VirtualGL/bin; sudo ./vglserver_config`.)
  8. Worked around a limitation in the AMDGPU drivers that prevented recent
  versions of Google Chrome from enabling GPU acceleration when used with
  VirtualGL.
* Mon Jul  6 2020 Ákos Szőts <szotsaki@gmail.com>
- update to 2.6.4
  * Changelog in packaged ChangeLog.md or at
    https://github.com/VirtualGL/virtualgl/blob/2.6.4/ChangeLog.md
- Refreshed patch VirtualGL-link-libs.patch
- Removed patch fix-Mesa-19.3.0-build.patch - resolved upstream
* Mon Jun  1 2020 Max Lin <mlin@suse.com>
- Remove conditional of applying glx.patch because we have newer Mesa
  in Leap 15.2
- Use upstream patch fix-Mesa-19.3.0-build.patch to replace glx.patch
  * Add fix-Mesa-19.3.0-build.patch
  * Remove glx.patch
* Sun Feb  9 2020 Adam Mizerski <adam@mizerski.pl>
- update to 2.6.3
  * Changelog in packaged ChangeLog.md or at
    https://github.com/VirtualGL/virtualgl/blob/2.6.3/ChangeLog.md
- Refreshed patch VirtualGL-link-libs.patch
- Removed patch virtualgl-nodl.patch - resolved upstream
- Added patch glx.patch
* Mon Nov  4 2019 Andreas Schwab <schwab@suse.de>
- Add riscv64 to the list of 64-bit architectures
* Tue Jun 19 2018 tchvatal@suse.com
- Do not provide the env files which change the preload order
  bsc#1097210
* Mon Dec  4 2017 tchvatal@suse.com
- Disable ssl feature as we can tunnel via ssh and the openssl-1.1
  is not compatible at all currently
* Mon Dec  4 2017 tchvatal@suse.com
- Version update 2.5.2:
  * Various compat fixes all around
- Remove merged gcc6_literal_suffixes.patch
- Refresh VirtualGL-link-libs.patch and virtualgl-nodl.patch
* Wed Aug 24 2016 tchvatal@suse.com
- Fix bnc#993645 as-needed eating symbols needed by this lib
  * virtualgl-nodl.patch
- Use system xcb headers
- Rename glxinfo2 to vglxinfo, sounds bit better and more descriptive
* Thu Jun 16 2016 tchvatal@suse.com
- Version update to 2.5 release
  * See upstream changelog for details
  * Mostly removal of compat for pre-VirtualGL namespace
- Refresh patch VirtualGL-link-libs.patch
* Thu Jun 16 2016 normand@linux.vnet.ibm.com
- new gcc6_literal_suffixes.patch
* Tue Sep  1 2015 tchvatal@suse.com
- Add proper wxwidgets license string
* Fri Aug 28 2015 tchvatal@suse.com
- Update to 2.4.1:
  * Various bugfixes not mentioned on changes
  * Refresh VirtualGL-link-libs.patch
* Fri Aug 29 2014 tchvatal@suse.com
- Version bump to beta1 of 2.4 (2.3.90)
- Fix building with -Wl,-z,now and no-undefined
  * VirtualGL-link-libs.patch
* Thu Aug 21 2014 dvaleev@suse.com
- Extend 64bit conditions to s390x ppc64 ppc64le aarch64
* Tue Jul  1 2014 tchvatal@suse.com
- Mention baselibs.conf in spec file
* Tue Jul  1 2014 tchvatal@suse.com
- Update license to WXwindows
* Mon Feb 10 2014 tchvatal@suse.com
- Require 32b libturbojpeg0 on the 32bit binary.
* Sun Feb  9 2014 tchvatal@suse.com
- No need to have different name for glxspheres on 64 and 32b.
* Fri Feb  7 2014 tchvatal@suse.com
- Use baselibs.conf
- Cleanup the spec to use suse macros
- Sort out dependencies and use system shared libjpeg-turbo
* Wed Nov 27 2013 arnaldo.coelho@gmail.com
- Update to 2.3.3
* Mon Oct 29 2012 arnaldo.coelho@gmail.com
- Update to 2.3.2
* Thu Sep  6 2012 arnaldo.coelho@gmail.com
- Update to 2.3.1
* Fri Dec 30 2011 bumblebee-project@hotmail.com
- Update to 2.3
* Mon Jul 11 2011 bumblebee-project@hotmail.com
- Initial package
